uniform int       num_lights;  // Number of lights to use
uniform sampler2D bump_tex;    // The bump-map 
uniform sampler2D detail_tex;  // The detail texture

uniform samplerCube t3d_shadow_tex;
uniform vec3        t3d_shadow_src;
uniform vec4        t3d_shadow_col;
uniform int         t3d_shadow_pass;

varying vec4      position;    // Object position of the vertex
varying vec3      camera_dir;  // Direction of the camera
varying mat3      tangent_mat; // The tangent matrix

// --------------------------------------------------------------------------
float falloff(vec3 light_dir,
              vec3 spot_dir,
              float cos_cutoff,
              float spot_exponent)
{
	const float cos_outer_cone_angle = 0.8; // 36 degrees 

	vec3 spot_direction = normalize(spot_dir);
	float cos_cur_angle = dot(spot_dir,-light_dir);
	float cos_inner_cone_angle = cos_cutoff;
	float cos_inner_minus_outer_angle =  cos_inner_cone_angle - cos_outer_cone_angle;
	float fall_off = clamp((cos_cur_angle - cos_outer_cone_angle) / cos_inner_minus_outer_angle,0.0,1.0);
	float spot = pow(cos_cur_angle,spot_exponent);
	return fall_off * spot;
}
// --------------------------------------------------------------------------
/*
vec4 shade_spot_light_fade(int light_index,
						         vec3 light_pos,
						         vec3 spot_dir,
						         vec3 camera_dir,
						         vec3 vertex,
						         vec3 normal)
{
	// Resulting color
   vec4 result = vec4(0.0);
    
	// Compute direction to the light & the distance
   vec3  light_dir = light_pos - vertex;
   float distance = length(light_dir);
   light_dir = normalize(light_dir);

   // Compute attenuation
   float att = 1.0 / (gl_LightSource[light_index].quadraticAttenuation * distance * distance + 
                      gl_LightSource[light_index].linearAttenuation    * distance + 
                      gl_LightSource[light_index].constantAttenuation);
                                  
	// Spotlight part
	if (gl_LightSource[light_index].position.w < 1.0) {
		att = att * falloff(
			light_dir,
			spot_dir,
			gl_LightSource[light_index].spotCosCutoff,
			gl_LightSource[light_index].spotExponent
		);
	}
	
	// Compute half vector
	vec3 half_vec = normalize(light_dir + camera_dir);
	
	// Compute diffuse & specular part
	float diffuse  = max(dot(light_dir,normal),0.0);
	float specular = pow(max(dot(normal,half_vec),0.0),gl_FrontMaterial.shininess);
	
   // Combine color
	result += gl_FrontMaterial.ambient * gl_LightSource[light_index].ambient;
	result += att * gl_FrontMaterial.diffuse * gl_LightSource[light_index].diffuse * diffuse;
	result += att * gl_FrontMaterial.specular * gl_LightSource[light_index].specular * specular;
	
	return result;
}
*/
// --------------------------------------------------------------------------
vec4 shade_spot_light_fade_bump(int light_index,
                                mat3 tangent_mat,
                                vec3 light_pos,
                                vec3 spot_dir,
                                vec3 camera_dir,
                                vec3 vertex,
                                vec3 normal)
{
	// Resulting color
   vec4 result = vec4(0.0);
    
	// Compute direction to the light & the distance
   vec3  light_dir = light_pos - vertex;
   float distance = length(light_dir);
   light_dir = normalize(light_dir);

   // Compute attenuation
   float att = 1.0 / (gl_LightSource[light_index].quadraticAttenuation * distance * distance + 
                      gl_LightSource[light_index].linearAttenuation    * distance + 
                      gl_LightSource[light_index].constantAttenuation);
                                  
	// Spotlight part
	if (gl_LightSource[light_index].position.w < 1.0) {
		att = max(att * falloff(
			light_dir,
			spot_dir,
			gl_LightSource[light_index].spotCosCutoff,
			gl_LightSource[light_index].spotExponent
		),0.0);
	}
	
	// Transform light direction to tangent space and compute half vector
	light_dir = normalize(light_dir * tangent_mat);
	vec3 half_vec = normalize(light_dir + camera_dir);
	
	// Compute diffuse & specular part
	float diffuse  = max(dot(light_dir,normal),0.0);
	float specular = pow(max(dot(normal,half_vec),0.0),gl_FrontMaterial.shininess);
	
	// Combine color
	result += gl_FrontMaterial.ambient * gl_LightSource[light_index].ambient;
	result += att * gl_FrontMaterial.diffuse * gl_LightSource[light_index].diffuse * diffuse;
	result += att * gl_FrontMaterial.specular * gl_LightSource[light_index].specular * specular;
	
	return result;
}
// --------------------------------------------------------------------------


vec4 do_shadow_pass()
{
	if (t3d_shadow_pass == 0) return vec4(0.0);

	// ERROR LIES HERE !
	vec4 result = vec4(0.0);
	
	vec3 light_pos = (gl_ModelViewMatrixInverse * gl_LightSource[0].position).xyz;
	vec3 spot_dir  = (gl_ModelViewMatrixInverse * vec4(gl_LightSource[0].spotDirection,0)).xyz;
		
	
	vec3 dir = normalize(position - t3d_shadow_src);
	vec4 cube_color = textureCube(t3d_shadow_tex,dir);
	
	if (cube_color.r >= 0.5) {
		result = t3d_shadow_col * falloff(
			normalize(light_pos - position),
			normalize(spot_dir),
			gl_LightSource[0].spotCosCutoff,
			gl_LightSource[0].spotExponent
		);
		result = vec4(1,0,0,1);
	} else {
		//discard;
		result = vec4(0.0);
	}
	
	//return result;
	
	return vec4(1.0);
}

vec4 do_normal_pass()
{
	//if (t3d_shadow_pass == 1) return vec4(0.0);

	vec4 result = vec4(0.0);
	
	// Get the detail texture color
	vec3 detail_col = vec3(texture2D(detail_tex,gl_TexCoord[0].xy));
	
	// Expand the bump-map into a normalized signed vector
	vec3 normal = vec3(texture2D(bump_tex,gl_TexCoord[0].xy));
	normal = 2.0 * (normal - 0.5);

	// Compute color using all lights
	vec4 color = vec4(0.0);
	for (int i=0; i<num_lights; i++) {

		// Compute light detail stuff
		vec3 light_pos = (gl_ModelViewMatrixInverse * gl_LightSource[i].position).xyz;
		vec3 spot_dir  = (gl_ModelViewMatrixInverse * vec4(gl_LightSource[i].spotDirection,0)).xyz;
		
		color += shade_spot_light_fade_bump(
			i,
			tangent_mat,
			light_pos,
			normalize(spot_dir),
			normalize(camera_dir),
			position,
			normalize(normal)
		);
	}
	
	// Combine texture color and light color
	result = color * vec4(detail_col,1.0);

	return result;
}


void main(void)
{
	gl_FragColor = do_normal_pass();// + do_shadow_pass();
}
