uniform vec3      t3d_camera_pos; // Camera position

uniform sampler2D disp_tex;       // Displacement texture
uniform float     density;        // Height densit

varying vec3      position;       // Object position of the vertex
varying vec3      camera_dir;     // Direction of the camera
varying mat3      tangent_mat;    // The tangent matrix

//attribute vec3 tangent;




#define textureSize 512.0
#define texelSize 1.0 / 512.0

vec4 texture2D_bilinear( uniform sampler2D tex, vec2 uv )
{
	vec2 f = fract( uv.xy * textureSize );
	vec4 t00 = texture2D( tex, uv );
	vec4 t10 = texture2D( tex, uv + vec2( texelSize, 0.0 ));
	vec4 tA = mix( t00, t10, f.x );
	vec4 t01 = texture2D( tex, uv + vec2( 0.0, texelSize ) );
	vec4 t11 = texture2D( tex, uv + vec2( texelSize, texelSize ) );
	vec4 tB = mix( t01, t11, f.x );
	return mix( tA, tB, f.y );
}
//--------------------------------------------------------------------------
void main(void)
{
	gl_TexCoord[0] = gl_MultiTexCoord0;
	
	// Lookup displacment
	//vec4 dv = texture2D(disp_tex,gl_TexCoord[0].xy);
	vec4 dv = texture2D_bilinear(disp_tex,gl_TexCoord[0].xy);
	
	// Compute new height
	float height = 0.299*dv.x + 0.587*dv.y + 0.114*dv.z;
	
	// Compute other stuff
	position    = vec3(normalize(gl_Normal) * height * density) + gl_Vertex;
	camera_dir  = normalize(t3d_camera_pos - position);
	gl_Position = gl_ModelViewProjectionMatrix * vec4(position,1.0);

	// Compute tangent matrix (gl_MultiTexCoord2 - tangent vectors)
	//
	//       [Tx, Ty, Tz]
	//   M = [Nx, Ny, Nz]
	//       [Bx, By, Bz]
	//
	vec3 n = normalize(gl_Normal);
	vec3 t = normalize(gl_MultiTexCoord2.xyz);
	vec3 b = normalize(cross(n,t));
	tangent_mat = mat3(t,b,n);
}
