
uniform bool OnlyDifference;
uniform bool EnableLighting;
uniform sampler2D EntryTex;
uniform sampler2D ExitTex;
uniform sampler3D DataTex;
uniform sampler1D TransferTex;

vec4 ComputeLightSpectrum(vec3 P,vec3 N)
{
	N = gl_NormalMatrix * N;

   vec3 L = normalize(gl_LightSource[0].position.xyz - P);
   vec3 E = normalize(-P);
   vec3 R = normalize(-reflect(L,N));

   vec4 Iamb  = gl_FrontLightProduct[0].ambient;
   vec4 Idiff = gl_FrontLightProduct[0].diffuse * max(dot(N,L), 0.0);
   vec4 Ispec = gl_FrontLightProduct[0].specular * pow(max(dot(R,E),0.0),0.3 * 30.0);
   
   return Iamb + Idiff + Ispec; 
}

vec4 Composite(vec3 point,vec4 result) 
{
	float intensity = texture3D(DataTex,vec3(point)).w;
	
   if (intensity <= 0.1) {
	return vec4(0.0,0.0,0.0,0.0);
   }
   
      vec4 transferred = texture1D(TransferTex,intensity);
      
      if (transferred.w != 0.0) {
      
         vec3 gradient = texture3D(DataTex,vec3(point)).xyz;

         // Compute gradient magnitude
         float gradMagnitude = sqrt(gradient.x * gradient.x + gradient.y * gradient.y + gradient.z * gradient.z);

         // Light computation
         //if (EnableLighting) {
            //result.xyz += (1.0 - result.w) * (transferred.xyz + ComputeLightSpectrum(point,normalize(gradient.xyz)).xyz) *  gradMagnitude;
            //result.w += (1.0 - result.w) * (transferred.w + gradMagnitude);
         //} else {
            //result.xyz += (1.0 - result.w) * transferred.xyz *  gradMagnitude;
            //result.w += (1.0 - result.w) * (transferred.w + gradMagnitude);
         //}
         //result.xyz += (1.0 - result.w) * transferred.xyz * transferred.w * gradMagnitude;
	result.xyz += (1.0 - result.w) * (transferred.xyz * ComputeLightSpectrum(point,normalize(gradient.xyz)).xyz);
         result.w += (1.0 - result.w) * transferred.w;
      }


   return result;
}

void main (void)
{  
   float dz = 10.0;

   vec4 entry_point = texture2D(EntryTex,vec2(gl_TexCoord[0]));
   vec4 exit_point  = texture2D(ExitTex, vec2(gl_TexCoord[0]));
   
   float dist = distance(entry_point,exit_point)/dz;
   int maxiter = int(floor(dist));
   vec3 diff = (exit_point.xyz - entry_point.xyz)/9.0;
   
  
   if (entry_point.xyz == vec3(0.0,0.0,0.0)) {
      discard;
   } else {
      vec3 point = entry_point.xyz;
      vec4 result = vec4(0.0,0.0,0.0,0.0);
      bool work = true;
      int i = 0;
      
      if (OnlyDifference) { 
      
			gl_FragColor =  abs(exit_point - entry_point);
      
      } else {
      
			/*
			while (work) {
				result = Composite(point,result);
				i++;
				point += diff;
				work = (i <= maxiter);
			}
	      
	      
			result = abs(exit_point - entry_point);
			result.w = 0.1;
	            
	      
	      result = Composite(point,result); point += diff;
			result = Composite(point,result); point += diff;
			result = Composite(point,result); point += diff;
			result = Composite(point,result); point += diff;
			result = Composite(point,result); point += diff;
	      
			result = Composite(point,result); point += diff;
			result = Composite(point,result); point += diff;*/
			result = Composite(point,result); point += diff;			
			if (result.w == 0.0) result = Composite(point,result); point += diff;
			if (result.w == 0.0) result = Composite(point,result); point += diff;
			
			if (result.w == 0.0) result = Composite(point,result); point += diff;
			//if (result.w == 0.0) result = Composite(point,result); point += diff;
			//if (result.w == 0.0) result = Composite(point,result); point += diff;
			//if (result.w == 0.0) result = Composite(point,result); point += diff;
			//if (result.w == 0.0) result = Composite(point,result); point += diff;
			
			result.w = 1.0;
			
			gl_FragColor = result;
		}
   }
}

