uniform samplerCube shadow_tex;
uniform int         shadow_light;
uniform vec3        shadow_src;
uniform vec4        shadow_col;

varying vec3        position;
varying vec3        normal;
varying vec3        light_pos;
varying vec3        spot_dir;

// --------------------------------------------------------------------------
float falloff(vec3 light_dir,
              vec3 spot_dir,
              float cos_cutoff,
              float spot_exponent)
{
	const float cos_outer_cone_angle = 0.8; // 36 degrees 

	vec3 spot_direction = normalize(spot_dir);
	float cos_cur_angle = dot(spot_dir,-light_dir);
	float cos_inner_cone_angle = cos_cutoff;
	float cos_inner_minus_outer_angle =  cos_inner_cone_angle - cos_outer_cone_angle;
	float fall_off = clamp((cos_cur_angle - cos_outer_cone_angle) / cos_inner_minus_outer_angle,0.0,1.0);
	float spot = pow(cos_cur_angle,spot_exponent);
	return fall_off * spot;
}
// --------------------------------------------------------------------------
float attenuate(int light_index,
                vec3 light_pos,
                vec3 spot_dir,
                vec3 vertex,
                vec3 normal)
{
	// Compute direction to the light & the distance
   vec3  light_dir = light_pos - vertex;
   float distance = length(light_dir);
   light_dir = normalize(light_dir);

   // Compute attenuation
   float att = 1.0 / (gl_LightSource[light_index].quadraticAttenuation * distance * distance + 
                      gl_LightSource[light_index].linearAttenuation    * distance + 
                      gl_LightSource[light_index].constantAttenuation);
                                  
	// Spotlight part
	if (gl_LightSource[light_index].position.w < 1.0) {
		att = max(att * falloff(
			light_dir,
			spot_dir,
			gl_LightSource[light_index].spotCosCutoff,
			gl_LightSource[light_index].spotExponent
		),0.0);
	}
	
	return att * max(dot(spot_dir,normal),0.0);
}
// --------------------------------------------------------------------------
void main (void)
{
	vec3 dir = normalize(position - light_pos);
	vec4 cube_color = textureCube(shadow_tex,dir);
	
	if (cube_color.r < 0.5) {
		discard;
	}
	
	// Attenuate shadow
	float att = attenuate(
		shadow_light,
		light_pos,
		normalize(spot_dir),
		position,
		normalize(normal)
	);

	// Compute final shadow
	gl_FragColor = att * shadow_col;
}
